@@ -131,16 +131,17 @@ class Dialcentral(object):
 
 		for button in (
 			"back",
+			"addressbookSelectButton",
 			"sendSmsButton",
+			"dialButton",
+			"selectButton",
 			"cancelSmsButton",
-			"sms_button",
-			"dial_button",
-			"select_button",
-			"cancel_button",
+			"callbackSelectButton",
 			"minutesEntryButton",
 			"clearcookies",
+			"phoneTypeSelection",
 		):
-			hildonize.set_button_thumb_selectable(button)
+			hildonize.set_button_thumb_selectable(self._widgetTree.get_widget(button))
 
 		replacementButtons = [gtk.Button("Test")]
 		menu = hildonize.hildonize_menu(
@@ -679,10 +680,11 @@ class Dialcentral(object):
 		"""
 		@note Hildon specific
 		"""
+		RETURN_TYPES = (gtk.keysyms.Return, gtk.keysyms.ISO_Enter, gtk.keysyms.KP_Enter)
 		try:
 			if (
 				event.keyval == gtk.keysyms.F6 or
-				event.keyval == gtk.keysyms.Return and event.get_state() & gtk.gdk.CONTROL_MASK
+				event.keyval in RETURN_TYPES and event.get_state() & gtk.gdk.CONTROL_MASK
 			):
 				if self._isFullScreen:
 					self._window.unfullscreen()
